@@ -17,6 +17,8 @@
 #include <sys/wait.h>
 #include <dirent.h>
 #include <errno.h>
+#include <time.h>
+
 #ifndef errno
 extern int errno;
 #endif
@@ -32,17 +34,23 @@ extern gshort dpmsmins;
 extern gboolean dpms;
 extern gboolean password;
 extern gchar *screensaver;
+extern const gchar *random_screensaver;
 extern screensaver_data *sd;
 extern GList *sdlist;
 static pid_t pid = 0;
 static pid_t pid_big = 0;
-
+static GtkWidget *item_list = NULL;
+static gint selected_item = 0;
 gboolean ignore_changes = FALSE;
+static gint random_x = 0;
+static gint random_y = 0;
+static guint random_timer = 0;
 
 /* Loading info... */
 void
 screensaver_load ()
 {
+        gint defval = 0;
         ss_priority = gnome_config_get_int ("/Screensaver/Default/nice=10");
         waitmins = gnome_config_get_int ("/Screensaver/Default/waitmins=20");
         if (waitmins > 9999)
@@ -56,7 +64,13 @@ screensaver_load ()
                 dpmsmins = 1;
         dpms = gnome_config_get_bool ("/Screensaver/Default/dpms=true");
         password = gnome_config_get_bool ("/Screensaver/Default/password=true");
-        screensaver = gnome_config_get_string ("/Screensaver/Default/mode=NONE");
+        screensaver = gnome_config_get_string_with_default ("/Screensaver/Default/mode", &defval);
+        if (defval) {
+                screensaver = random_screensaver;
+        } else if (!strcmp (screensaver, random_screensaver)) {
+                g_free (screensaver);
+                screensaver = random_screensaver;
+        }
         sd = NULL;
 }
 
@@ -149,21 +163,37 @@ get_and_set_dpmscheck (GtkWidget *box)
 
 static gint compareItems(const void *a, const void *b)
 {
-        gint res;
-        
-        return strcmp((char *)b, (char *)a);
+        return strcmp((char *)a, (char *)b);
 }
+static screensaver_data *
+get_random_data ()
+{
+        screensaver_data *retval;
 
+        retval = g_malloc (sizeof (screensaver_data));
+        retval->name = random_screensaver;
+        retval->tryexec = NULL;
+        retval->desktop_filename = NULL;
+        retval->windowid = NULL;
+        retval->root = NULL;
+        retval->args = NULL;
+        retval->icon = NULL;
+        retval->comment = NULL;
+        retval->author = NULL;
+        retval->demo = NULL;
+        retval->dialog = NULL;
+        retval->setup_data = NULL;
+        return retval;
+}
 void
-create_list (GtkList *list, gchar *directory)
+create_list (GtkCList *list, gchar *directory)
 {
         DIR *dir;
         gchar *label;
-        GtkWidget *list_item = NULL;
-        GtkWidget *temp_item;
+        gchar *list_item[1];
+        gint item;
         struct dirent *child;
         gchar *prefix;
-        GList *items = NULL;
         GList *desktop_items = NULL, *listp;
         screensaver_data *sdnew;
 
@@ -178,19 +208,19 @@ create_list (GtkList *list, gchar *directory)
                 if (child->d_name[0] != '.') {
                         desktop_items = g_list_insert_sorted(desktop_items, 
                                              g_strdup(child->d_name),
-                                             (GCompareFunc)compareItems); 
+                                             (GCompareFunc)strcmp); 
                 }
         }
 
-        listp = desktop_items;
-        while (listp) {
+        for (listp = desktop_items; listp; listp = g_list_next(listp)) {
                 gchar *name;
 
                 name = listp->data;
                 prefix = g_strconcat ("=", directory, name, "=/Desktop Entry/", NULL);
                 gnome_config_push_prefix (prefix);
                 g_free (prefix);
-                
+
+                /* Lets make the data for the screensaver */
                 sdnew = g_malloc (sizeof (screensaver_data));
                 sdnew->desktop_filename = g_strconcat (directory, name, NULL);
                 sdnew->name = gnome_config_get_translated_string ("Name");
@@ -227,93 +257,117 @@ create_list (GtkList *list, gchar *directory)
                                 /* bah -- useless file... */
                         break;
                 }
-                temp_item = gtk_list_item_new_with_label (sdnew->name);
-                items = g_list_prepend (items, temp_item);
-                if (strcmp (sdnew->name, screensaver) == 0) {
+                list_item[0] = sdnew->name;
+                item = gtk_clist_append (list, list_item);
+                if (screensaver && strcmp (sdnew->name, screensaver) == 0) {
                         sd = sdnew;
-                        list_item = temp_item;
                         init_screensaver_data (sdnew);
+                        selected_item = item;
                         if (sdnew->setup_data == NULL)
                                 gtk_widget_set_sensitive (setup_button, FALSE);
                 }
-                gtk_object_set_data(GTK_OBJECT(items->data), "sdata",
-                                    sdnew);
-/*                        gtk_signal_connect (GTK_OBJECT (items->data),
-                          "button_press_event",
-                          (GtkSignalFunc) list_click_callback,
-                          sdnew);
-*/
-
-                listp = g_list_next(listp);
+                gtk_clist_set_row_data(list, item, sdnew);
         }
 
         if (desktop_items) {
                 g_list_foreach(desktop_items, (GFunc)g_free, NULL);
                 g_list_free(desktop_items);
         }
-
-        if (items) {
-                gtk_list_append_items (list, items);
-                if (list_item) {
-                        gtk_list_select_child (list, list_item);
-                }
-                else
-                        gtk_widget_set_sensitive (setup_button, FALSE);
-        } 
-                
+        if (selected_item) {
+                gtk_clist_select_row (list, selected_item, 0);
+        }
 }
 
-GtkWidget
-*get_and_set_mode()
+GtkWidget *
+get_and_set_mode()
 {
-        GList *templist = NULL;
-        //   screensaver_data *random;
+        screensaver_data *random;
         gchar *tempdir;
+        gchar *list_item[1];
+        GtkWidget *swindow;
 
-        static GtkWidget *list = NULL;
-
-        if (!list) {
-                list = gtk_list_new();
-                gtk_list_set_selection_mode (GTK_LIST (list), GTK_SELECTION_BROWSE);
-                templist = g_list_prepend (templist, gtk_list_item_new_with_label ("No Screensaver"));
-                gtk_list_prepend_items (GTK_LIST (list), templist);
-
-                //random = get_random_data ();
-
+        if (!item_list) {
+                swindow = gtk_scrolled_window_new (NULL, NULL);
+                gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
                 
+                item_list = gtk_clist_new (1);
+                gtk_clist_set_column_auto_resize (GTK_CLIST (item_list), 0, TRUE);
+                gtk_clist_set_selection_mode (GTK_CLIST (item_list), GTK_SELECTION_BROWSE);
+                list_item [0] = _("No Screensaver");
+                gtk_clist_append (GTK_CLIST (item_list), list_item);
+                gtk_container_add (GTK_CONTAINER (swindow), item_list);
+
+                random = get_random_data ();                
+                list_item [0] = _("Random Screensaver");
+                gtk_clist_append (GTK_CLIST (item_list), list_item);
+                gtk_clist_set_row_data (GTK_CLIST (item_list), 1, random);
+                if (screensaver == random_screensaver) {
+                        /* In the case that it's the */
+                        sd = random;
+                        selected_item = 1;
+                        gtk_widget_set_sensitive (setup_button, FALSE);
+                } else if (!strcmp (screensaver, "NONE")) {
+                        sd = NULL;
+                        selected_item = 0;
+                        gtk_widget_set_sensitive (setup_button, FALSE);
+                }                
                 tempdir = gnome_unconditional_datadir_file ("control-center/.data/");
-                create_list (GTK_LIST (list), tempdir);
+                create_list (GTK_CLIST (item_list), tempdir);
                 g_free (tempdir);
-                tempdir = gnome_util_home_file ("control-center/.data/");
-                create_list (GTK_LIST (list), tempdir);
-                g_free (tempdir);
-                gtk_signal_connect (GTK_OBJECT (list), 
-                                    "selection_changed",
+                gtk_signal_connect (GTK_OBJECT (item_list), 
+                                    "select_row",
                                     (GtkSignalFunc) list_click_callback, 
-                                    NULL); 
+                                    NULL);
+                return swindow;
         } else {
                 gint i=0;
                 GList *dlist = NULL;
 
-                screensaver = gnome_config_get_string ("/Screensaver/Default/mode=NONE");
-
-                dlist = gtk_container_children(GTK_CONTAINER(list));
-                while (dlist) {
-                        gchar *str;
-
-                        gtk_label_get(GTK_LABEL(GTK_BIN(dlist->data)->child),
-                                &str);
-                        if (strcmp(str, screensaver) == 0) 
+                screensaver = gnome_config_get_string_with_default ("/Screensaver/Default/mode=RANDOM SCREENSAVER", &i);
+                /* The default, we know is the Random screensaver */
+                if (i) {
+                        gtk_clist_select_row (GTK_CLIST (item_list), 1, 0);
+                        gtk_clist_moveto (GTK_CLIST (item_list), 1, 0, 0.5, 0.0);
+                        return NULL;
+                }
+                if (!strcmp (screensaver, "NONE")) {
+                        gtk_clist_select_row (GTK_CLIST (item_list), 0, 0);
+                        gtk_clist_moveto (GTK_CLIST (item_list), 0, 0, 0.5, 0.0);
+                        return NULL;
+                }
+                /* Damn, we need to find the stupid thing.. (-: */
+                for (dlist = GTK_CLIST (item_list)->row_list; dlist; dlist = dlist->next) {
+                        if (!strcmp ( screensaver,
+                                       GTK_CELL_TEXT (GTK_CLIST_ROW (dlist)->cell[0])->text)) {
+                                gtk_clist_moveto (GTK_CLIST (item_list), i, 0, 0.5, 0.0);
+                                gtk_clist_select_row (GTK_CLIST (item_list), i, 0);
                                 break;
-                        dlist = g_list_next(dlist);
+                        }
                         i++;
                 }
-                if (dlist)
-                        gtk_list_select_item(GTK_LIST(list), i);
-
         }
+        return NULL;
+}
+static gint
+timer_cb ()
+{
+        static gint length = 0;
 
-                return list;
+        if (!length)
+                length = gdk_string_width (monitor->style->font, random_screensaver);
+        random_x = (gint) (((gfloat)monitor->allocation.width - length)*random()/(RAND_MAX+1.0));
+        random_y = monitor->style->font->descent + (gint) (((gfloat)monitor->allocation.height - monitor->style->font->descent)*random()/(RAND_MAX+1.0));
+
+        gdk_draw_rectangle (monitor->window,monitor->style->black_gc, TRUE, 0, 0, monitor->allocation.width, monitor->allocation.height);
+        gdk_draw_text (monitor->window,
+                       monitor->style->font,
+                       monitor->style->white_gc, 
+                       random_x,
+                       random_y,
+                       _(random_screensaver),
+                       strlen (_(random_screensaver)));
+        
+        return 1;
 }
 
 void
@@ -328,7 +382,36 @@ launch_miniview (screensaver_data *sd)
                 kill (pid, SIGTERM);
                 pid = 0;
         }
+        waitpid(pid, &pid, 0);
+        if (sd && sd->name == random_screensaver) {
+                gint length = gdk_string_width (monitor->style->font, random_screensaver);
+                guint seed;
 
+                seed = time (NULL);
+                srandom (seed);
+
+                random_x = (monitor->allocation.width-length)/2;
+                random_y = monitor->allocation.height/2;
+                gdk_draw_rectangle (monitor->window,monitor->style->black_gc, TRUE, 0, 0, monitor->allocation.width, monitor->allocation.height);
+                gdk_draw_text (monitor->window,
+                               monitor->style->font,
+                               monitor->style->white_gc,
+                               random_x,
+                               random_y,
+                               _(random_screensaver),
+                               strlen (_(random_screensaver)));
+                random_timer = gtk_timeout_add (3000, (GtkFunction) (timer_cb), NULL); 
+                return;
+        }
+        if (random_timer) {
+                /* removing the timer */
+                gtk_timeout_remove (random_timer);
+                random_timer = 0;
+        }
+        if (sd == NULL) {
+                gdk_draw_rectangle (monitor->window,monitor->style->black_gc, TRUE, 0, 0, monitor->allocation.width, monitor->allocation.height);
+                return;
+        }
         if (sd->demo == NULL)
                 /* FIXME: um, we _should_ do something.  Maybe steal the icon field.  */
                 /* Maybe come up with a default demo.  dunno... */
@@ -380,26 +463,30 @@ setup_callback (GtkWidget *widget, gpointer data)
                 sd->dialog = make_dialog (sd);
         }
 }
-void
+static void
 handle_list_change(void *data)
 {
         gchar *label;
 
         if (sd == data)
                 return;
-        if (data == NULL){
+        if (data == NULL) {
                 screensaver = NULL;
                 sd = NULL;
                 gtk_label_set (GTK_LABEL (setup_label), _("Settings..."));                
                 gtk_widget_set_sensitive (setup_button, FALSE);
-                if (pid) {
-                        kill (pid, SIGTERM);
-                        pid = 0;
-                }
-                gdk_draw_rectangle (monitor->window,monitor->style->black_gc, TRUE, 0, 0, monitor->allocation.width, monitor->allocation.height);
+                launch_miniview (sd);
                 return;
         }
         sd = (screensaver_data *) data;
+        if (sd->name == random_screensaver) {
+                /* we need to special case random */
+                gtk_label_set (GTK_LABEL (setup_label),_("Random Settings"));
+                gtk_widget_set_sensitive (setup_button, FALSE);
+                launch_miniview (sd);
+                screensaver = sd->name;
+                return;
+        }
         if (sd->args == NULL)
                 init_screensaver_data (sd);
         label = g_strconcat (sd->name, _(" Settings"), NULL);
@@ -414,25 +501,39 @@ handle_list_change(void *data)
         screensaver = sd->name;
 }
 void
-list_click_callback (GtkWidget *widget,  void *data)
+list_click_callback (GtkWidget *widget,
+                     gint row,
+                     gint column,
+                     GdkEventButton *event,
+                     gpointer data)
 {
-        GList *temp = GTK_LIST (widget)->selection;
-        if (!temp)
-                return;
         if (!ignore_changes)
                 capplet_widget_state_changed(CAPPLET_WIDGET (capplet), TRUE);
-        handle_list_change(gtk_object_get_data(GTK_OBJECT(temp->data), "sdata"));
+        handle_list_change(gtk_clist_get_row_data(GTK_CLIST (widget), row));
 }
 
 void
-list_expose_callback(GtkWidget *widget, GdkEventButton *event, void *data)
+monitor_expose_callback (GtkWidget *monitor, GdkEventButton *event, void *data)
 {
-        gtk_signal_disconnect_by_func (GTK_OBJECT (widget), list_expose_callback, data);
+        static gint length = 0;
+
+        if (length == 0) {
+                gdk_draw_rectangle (monitor->window,monitor->style->black_gc, TRUE, 0, 0, monitor->allocation.width, monitor->allocation.height);
+                handle_list_change(data);
+                length = gdk_string_width (monitor->style->font, random_screensaver);
+                gtk_clist_moveto (GTK_CLIST (item_list), selected_item, 0, 0.5, 0.0);
+        }
+        if (!sd || sd->name != random_screensaver)
+                return;
         gdk_draw_rectangle (monitor->window,monitor->style->black_gc, TRUE, 0, 0, monitor->allocation.width, monitor->allocation.height);
-        handle_list_change(data);
+        gdk_draw_text (monitor->window,
+                       monitor->style->font,
+                       monitor->style->white_gc,
+                       random_x,
+                       random_y,
+                       _(random_screensaver),
+                       strlen (_(random_screensaver)));
 }
-
-
 void
 insert_text_callback (GtkEditable    *editable, const gchar    *text,
                            gint length, gint *position,
@@ -494,16 +595,21 @@ nice_callback (GtkObject *adj, void *data)
                 capplet_widget_state_changed(CAPPLET_WIDGET (capplet), TRUE);
 }
 static void
-ssaver_callback (GtkWidget *dialog, GdkEventKey *event, GdkWindow *sswin)
+ssaver_callback (GtkWidget *dialog, GdkEvent *event, GdkWindow *sswin)
 {
-        
         if (pid_big){
-                kill (pid_big, SIGTERM);
+                kill (pid_big, SIGKILL);
                 pid_big = 0;
+#if 0
+                if (event->any.type == GDK_KEY_PRESS)
+                        gtk_signal_emit_stop_by_name (GTK_OBJECT (dialog),"key_press_event");
+                else if (event->any.type == GDK_BUTTON_PRESS)
+                        gtk_signal_emit_stop_by_name (GTK_OBJECT (dialog),"button_press_event");
+#endif
+                if (sswin) {
+                        gdk_window_hide (sswin);
+                }
         }
-        /*gtk_signal_emit_stop_by_name (GTK_OBJECT (dialog),"key_press_event");
-          gtk_signal_emit_stop_by_name (GTK_OBJECT (dialog),"button_press_event");*/
-        gdk_window_hide (sswin);
 }
 void
 dialog_destroy_callback (GtkWidget *dialog, screensaver_data *newsd)
@@ -537,7 +643,7 @@ dialog_callback (GtkWidget *dialog, gint button, screensaver_data *newsd)
                         attributes.override_redirect = TRUE;
                         sswin = gdk_window_new (NULL, &attributes, GDK_WA_X|GDK_WA_Y|GDK_WA_NOREDIR);
                 }
-
+                g_print ("setting it all up\n");
                 gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event", (GtkSignalFunc) ssaver_callback, sswin);
                 gtk_signal_connect (GTK_OBJECT (dialog), "button_press_event", (GtkSignalFunc) ssaver_callback, sswin);
                 XSelectInput (GDK_WINDOW_XDISPLAY (sswin), GDK_WINDOW_XWINDOW (sswin), ButtonPressMask | KeyPressMask);
@@ -556,9 +662,9 @@ dialog_callback (GtkWidget *dialog, gint button, screensaver_data *newsd)
                         temp = g_strconcat (newsd->args, " ", newsd->windowid, " ", xid, NULL);
 #endif
                         argv = g_strsplit (temp, " ", -1);
-                        g_free (temp);
-                        temp = gnome_is_program_in_path (argv[0]);
-                        execvp (temp, argv);
+
+                        execvp (argv[0], argv);
+                        exit (1);
                         /* This call should never return */
                 }
                 break;
@@ -582,6 +688,8 @@ ok_callback ()
         /* we want to commit our changes to disk. */
         GString *command;
         gchar temp[10];
+        gchar argv[3];
+        gint temp2;
 
 
         /* save the stuff... */
@@ -595,9 +703,7 @@ ok_callback ()
         else
                 gnome_config_set_string ("/Screensaver/Default/mode", "NONE");
 
-        gnome_config_sync ();
         system ("xscreensaver-command -exit");
-        
         if (sd && waitmins) {
                 if (sd->dialog)
                         store_screensaver_data (sd);
@@ -609,22 +715,27 @@ ok_callback ()
                 g_string_append (command, temp);
                 if (password)
                         g_string_append (command, " -lock-mode");
-                g_string_append (command," -xrm \"*programs:\t");
+                if (sd->name != random_screensaver) {
+                        g_string_append (command," -xrm \"*programs:\t");
 #ifdef HAVE_REDHAT_XSCREENSAVER_RPM
-                g_string_append (command, "/usr/X11R6/lib/xscreensaver/");
+                        g_string_append (command, "/usr/X11R6/lib/xscreensaver/");
 #endif
+                }
                 if (sd->args && *(sd->args))
                         g_string_append (command, sd->args);
-                else
+                else if (sd->tryexec)
                         g_string_append (command, sd->tryexec);
                 if (sd->root) {
                         g_string_append (command, " ");
                         g_string_append (command, sd->root);
                 }
-                g_string_append (command, "\"&");
-                system (command->str);
+                if (sd->name != random_screensaver)
+                        g_string_append (command, "\"");
                 gnome_config_set_string ("/Screensaver/Default/command", command->str);
-                
+                g_string_append (command, " &");
+                g_print ("\n*\n*\n*\n*running:%s:\n*\n*\n*\n", command->str);
+                system (command->str);
+                g_print ("done........\n");
                 g_string_free (command, TRUE);
         }
         if (dpms) {
@@ -644,13 +755,41 @@ ok_callback ()
         else
                 system ("xset -dpms");
 
+        gnome_config_sync ();
+}
+void
+page_hide_callback ()
+{
+        if (sd->dialog)
+                gtk_widget_hide (sd->dialog);
+}
+void
+page_show_callback ()
+{
+        if (sd->dialog)
+                gtk_widget_show (sd->dialog);
+}
+void
+help_callback () {
+
+  gchar *tmp;
+
+  tmp = gnome_help_file_find_file ("users-guide", "gccdesktop.html#GCCSCREEN");
+  if (tmp) {
+    gnome_help_goto(0, tmp);
+    g_free(tmp);
+  }
+
 }
 void
 try_callback ()
 {
         GString *command;
         gchar temp[10];
+        
+        g_print ("nuking xscreensaver2\n");
         system ("xscreensaver-command -exit");
+
         
         if (sd && waitmins) {
                 if (sd->dialog)
@@ -663,16 +802,23 @@ try_callback ()
                 g_string_append (command, temp);
                 if (password)
                         g_string_append (command, " -lock-mode");
-                g_string_append (command," -xrm \"*programs:\t");
+                if (sd->name != random_screensaver) {
+                        g_string_append (command," -xrm \"*programs:\t");
+#ifdef HAVE_REDHAT_XSCREENSAVER_RPM
+                        g_string_append (command, "/usr/X11R6/lib/xscreensaver/");
+#endif
+                }
                 if (sd->args && *(sd->args)) 
                         g_string_append (command, sd->args);
-                else
+                else if (sd->tryexec)
                         g_string_append (command, sd->tryexec);
                 if (sd->root) {
                         g_string_append (command, " ");
                         g_string_append (command, sd->root);
                 }
-                g_string_append (command, "\"&");
+                if (sd->name != random_screensaver)
+                        g_string_append (command, "\"");
+                g_string_append (command, "&");
                 
                 system (command->str);
                 g_string_free (command, TRUE);
@@ -709,15 +855,17 @@ revert_callback ()
         gnome_config_drop_all ();
         screensaver_load ();
 
-        /* try to kill any existing xscreensaver */
-        system ("xscreensaver-command -exit");
         if (pid) {
                 kill (pid, SIGTERM);
                 pid = 0;
         }
+        waitpid(pid, &pid, 0);
+        /* try to kill any existing xscreensaver */
+        system ("xscreensaver-command -exit");
+        
         gdk_draw_rectangle (monitor->window,monitor->style->black_gc, TRUE, 0, 0, monitor->allocation.width, monitor->allocation.height);
 
-        temp = gnome_config_get_string ("/Screensaver/Default/command=xscreensaver");
+        temp = gnome_config_get_string ("/Screensaver/Default/command=xscreensaver -no-splash -timeout 20 -nice 10 -lock-mode");
         if (!strchr(temp, '&')) {
                 /* ugh! */
                 temp2 = g_strconcat (temp, " &", NULL);
