@@ -1,15 +1,15 @@
 #include "da.h"
+#include <errno.h>
 
 GtkWidget *plug;
 
-gint pid;
 void
 send_socket()
 {
   gchar buffer[256];
-  
-  g_snprintf(buffer, sizeof(buffer), "%11x ", 
-	     GDK_WINDOW_XWINDOW (preview_socket->window));
+
+  g_snprintf(buffer, sizeof(buffer), "%11lx ", 
+	     (gulong)GDK_WINDOW_XWINDOW (preview_socket->window));
   write(prog_fd, buffer, strlen(buffer));
 }
 
@@ -17,28 +17,31 @@ void
 send_reread()
 {
   gchar buffer[256];
-  
+
   g_snprintf(buffer, sizeof(buffer), "R ");
   write(prog_fd, buffer, strlen(buffer));
-  fsync(prog_fd);
 }
 
-void
+static void
 demo_data_in(gpointer data, gint source, GdkInputCondition condition)
 {
   gchar buf[256];
   
-  read(0, buf, 2);
-  if (gtk_rc_reparse_all ())
-    gtk_widget_reset_rc_styles(plug);
+  if (read(source, buf, 2) == 0)
+    gtk_main_quit();		/* Parent exited */
+  else {
+    if (gtk_rc_reparse_all ())
+      gtk_widget_reset_rc_styles(plug);
+  }
 }
 
 #define NUM 50
 
-void demo_main(int argc, char **argv)
+static void
+demo_main(int argc, char **argv, gint in_fd)
 {
   gchar buf[256];
-  XID window;
+  Window window;
   GtkWidget *widget, *table, *hbox;
   GtkWidget *scrolled_window;
   GSList *group;
@@ -54,9 +57,12 @@ void demo_main(int argc, char **argv)
   gchar *home_dir;
   gint i;
 
-  read(0, buf, 12);
+  if (read(in_fd, buf, 12) <= 0)
+    /* Assume this means that our parent exited or was killed */
+    exit(0);
+  
   buf[12] = 0;
-  sscanf(buf, "%x", &window);
+  window = strtol (buf, NULL, 16);
 
   fcntl(0, F_SETFL, O_NONBLOCK);
 
@@ -149,34 +155,35 @@ void demo_main(int argc, char **argv)
 
   gtk_container_add (GTK_CONTAINER (scrolled_window), widget);
   
-  gdk_input_add_full(0, GDK_INPUT_READ, demo_data_in, NULL, NULL);
+  gdk_input_add_full(in_fd, GDK_INPUT_READ, demo_data_in, NULL, NULL);
   gtk_widget_show_all (plug);
   
   gtk_main ();
 }
 
-void
+gint
 do_demo(int argc, char **argv)
 {
   gint toProg[2];
+  gint pid;
   
   pipe(toProg);
   
   if (!(pid = fork()))
     {
       close(toProg[1]);
-      dup2(toProg[0], 0);   /* Make stdin the in pipe */
-      close(toProg[0]);
-      demo_main(argc, argv);
+      demo_main(argc, argv, toProg[0]);
+      exit(0);
     }
   else if (pid > 0)
     {
       close(toProg[0]);
       prog_fd = toProg[1];
+      return pid;
     }
   else
     {
       /* baaaaaaaah eeeeek */
+      return -1;
     }
-  
 }
