@@ -14,7 +14,10 @@ enum {
 	REVERT_SIGNAL,
 	OK_SIGNAL,
         CANCEL_SIGNAL,
+        HELP_SIGNAL,
         NEW_MULTI_CAPPLET,
+        PAGE_HIDDEN_SIGNAL,
+        PAGE_SHOWN_SIGNAL,
         LAST_SIGNAL
 };
 static int capplet_widget_signals[LAST_SIGNAL] = {0,0,0,0};
@@ -29,6 +32,8 @@ void _capplet_widget_server_ok(gint id);
 void _capplet_widget_server_cancel(gint id);
 void _capplet_widget_server_help(gint id);
 void _capplet_widget_server_new_multi_capplet(gint id, gint capid);
+void _capplet_widget_server_page_hidden(gint id);
+void _capplet_widget_server_page_shown(gint id);
 
 
 /* administrative calls */
@@ -105,7 +110,7 @@ capplet_widget_class_init (CappletWidgetClass *klass)
 			       			 cancel),
                                gtk_marshal_NONE__NONE,
                                GTK_TYPE_NONE, 0);
-  	capplet_widget_signals[CANCEL_SIGNAL] =
+  	capplet_widget_signals[HELP_SIGNAL] =
 		gtk_signal_new("help",
 			       GTK_RUN_LAST,
 			       object_class->type,
@@ -121,6 +126,22 @@ capplet_widget_class_init (CappletWidgetClass *klass)
 			       			 new_multi_capplet),
                                gtk_marshal_NONE__POINTER,
                                GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+  	capplet_widget_signals[PAGE_HIDDEN_SIGNAL] =
+		gtk_signal_new("page_hidden",
+			       GTK_RUN_LAST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET(CappletWidgetClass,
+			       			 page_hidden),
+                               gtk_marshal_NONE__NONE,
+                               GTK_TYPE_NONE, 0);
+  	capplet_widget_signals[PAGE_SHOWN_SIGNAL] =
+		gtk_signal_new("page_shown",
+			       GTK_RUN_LAST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET(CappletWidgetClass,
+			       			 page_shown),
+                               gtk_marshal_NONE__NONE,
+                               GTK_TYPE_NONE, 0);
 
         gtk_object_class_add_signals (object_class, capplet_widget_signals, LAST_SIGNAL);
         klass->try = NULL;
@@ -236,6 +257,18 @@ _capplet_widget_server_new_multi_capplet(gint id, gint capid)
         GtkWidget *capplet = get_widget_by_id (id);
         gtk_signal_emit_by_name(GTK_OBJECT (capplet) ,"new_multi_capplet", capplet_widget_multi_new (capid));
 }
+void
+_capplet_widget_server_page_hidden(gint id)
+{
+        GtkWidget *capplet = get_widget_by_id (id);
+        gtk_signal_emit_by_name(GTK_OBJECT (capplet) ,"page_hidden");
+}
+void
+_capplet_widget_server_page_shown(gint id)
+{
+        GtkWidget *capplet = get_widget_by_id (id);
+        gtk_signal_emit_by_name(GTK_OBJECT (capplet) ,"page_shown");
+}
 static GtkWidget *
 get_widget_by_id(gint id)
 {
